import { app, sequelize } from '@/app.ts';
import { Org } from '@/models/org.ts';
import { User } from '@/models/user.ts';
import { CustomError } from '@kevisual/router';
import { Op } from 'sequelize';

app
  .route({
    path: 'org',
    key: 'list',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const list = await Org.findAll({
      order: [['updatedAt', 'DESC']],
      where: {
        users: {
          [Op.contains]: [
            {
              uid: tokenUser.id,
            },
          ],
        },
      },
      logging: false,
    });

    ctx.body = list;
    return ctx;
  })
  .addTo(app);

app
  .route({
    path: 'org',
    key: 'update',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { username, description, id } = ctx.query.data;
    if (!username) {
      throw new CustomError('username is required');
    }
    if (id) {
      const org = await Org.findByPk(id);
      if (!org) {
        throw new CustomError('org not found');
      }
      org.description = description;
      await org.save();
      const user = await User.findOne({ where: { username } });
      user.description = description;
      await user.save();
      ctx.body = {
        id: user.id,
        username: user.username,
        description: user.description,
      };
      return;
    }
    const user = await User.createOrg(username, tokenUser.id, description);
    ctx.body = {
      id: user.id,
      username: user.username,
      description: user.description,
    };
  })
  .addTo(app);

app
  .route({
    path: 'org',
    key: 'delete',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const id = ctx.query.id;
    if (!id) {
      throw new CustomError('id is required');
    }
    const org = await Org.findByPk(id);
    if (!org) {
      throw new CustomError('org not found');
    }
    const username = org.username;
    const users = org.users;
    const owner = users.find((u) => u.role === 'owner');
    if (owner.uid !== tokenUser.id) {
      throw new CustomError('Permission denied');
    }
    await org.destroy({ force: true });
    const orgUser = await User.findOne({
      where: { username },
    });
    await orgUser.destroy({ force: true });
    ctx.body = 'success';
  })
  .addTo(app);

app
  .route({
    path: 'org',
    key: 'get',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const id = ctx.query.id;
    if (!id) {
      throw new CustomError('id is required');
    }
    const org = await Org.findByPk(id);
    if (!org) {
      throw new CustomError('org not found');
    }
    const usersIds = org.users;
    const me = usersIds.find((u) => u.uid === tokenUser.id);
    if (!me) {
      throw new CustomError('Permission denied');
    }
    const _users = await User.findAll({
      where: {
        id: {
          [Op.in]: usersIds.map((u) => u.uid),
        },
      },
    });

    const users = _users.map((u) => {
      const role = usersIds.find((r) => r.uid === u.id)?.role;
      return {
        id: u.id,
        username: u.username,
        role: role,
      };
    });
    ctx.body = {
      org,
      users,
    };
  })
  .addTo(app);
